/*******************************************************************************************
*
*   raylib - sample game: snake
*
*   Sample game developed by Ian Eito, Albert Martos and Ramon Santamaria
*
*   This game has been created using raylib v1.3 (www.raylib.com)
*   raylib is licensed under an unmodified zlib/libpng license (View raylib.h for details)
*
*   Copyright (c) 2015 Ramon Santamaria (@raysan5)
*
********************************************************************************************/

#include "raylib.h"

#if defined(PLATFORM_WEB)
    #include <emscripten/emscripten.h>
#endif

//----------------------------------------------------------------------------------
// Some Defines
//----------------------------------------------------------------------------------
#define SNAKE_LENGTH   256
#define SQUARE_SIZE     31
#define NUMBEROFWALL 8



//----------------------------------------------------------------------------------
// Types and Structures Definition
//----------------------------------------------------------------------------------
typedef struct Snake {
    Vector2 position;
    Vector2 size;
    Vector2 speed;
    int score;
    Color color;
} Snake;

typedef struct Food {
    Vector2 position;
    Vector2 size;
    bool active;
    Color color;
} Food;

//------------------------------------------------------------------------------------
// Global Variables Declaration
//------------------------------------------------------------------------------------
static const int screenWidth = 800;
static const int screenHeight = 450;

static int framesCounter = 0;
static bool gameOver = false;
static bool pause = false;

static Food fruit = { 0 };
static Snake snake[SNAKE_LENGTH] = { 0 };
static Vector2 snakePosition[SNAKE_LENGTH] = { 0 };
static bool allowMove = false;
static Vector2 offset = { 0 };
static int counterTail = 0;

static Food goldenFruit = {0};
static Food deadFruit = {0};

static Rectangle recTab[NUMBEROFWALL];
static int highscore;
static Sound fruitSound ;     
static Sound goldenFruitSound;
static Sound deathSound; 
static Music music;



//------------------------------------------------------------------------------------
// Module Functions Declaration (local)
//------------------------------------------------------------------------------------
static void InitGame(void);         // Initialize game
static void UpdateGame(void);       // Update game (one frame)
static void DrawGame(void);         // Draw game (one frame)
static void UnloadGame(void);       // Unload game
static void UpdateDrawFrame(void);  // Update and Draw (one frame)

//------------------------------------------------------------------------------------
// Program main entry point
//------------------------------------------------------------------------------------
int main(void)
{
    // Initialization (Note windowTitle is unused on Android)
    //---------------------------------------------------------
    InitWindow(screenWidth, screenHeight, "sample game: snake");

    InitGame();
    InitAudioDevice(); // Initialize audio device
    StopMusicStream(music);
    PlayMusicStream(music);    

    

#if defined(PLATFORM_WEB)
    emscripten_set_main_loop(UpdateDrawFrame, 0, 1);
#else
    SetTargetFPS(60);
    //--------------------------------------------------------------------------------------

    // Main game loop
    while (!WindowShouldClose())    // Detect window close button or ESC key
    {
        // Update and Draw
        //----------------------------------------------------------------------------------
        UpdateDrawFrame();
        //----------------------------------------------------------------------------------
    }
#endif
    // De-Initialization
    //--------------------------------------------------------------------------------------
    UnloadGame();         // Unload loaded data (textures, sounds, models...)
    
    CloseWindow();        // Close window and OpenGL context
    //--------------------------------------------------------------------------------------

    return 0;
}

//------------------------------------------------------------------------------------
// Module Functions Definitions (local)
//------------------------------------------------------------------------------------

// Initialize game variables
void InitGame(void)
{
    framesCounter = 0;
    snake[0].score=0;
    gameOver = false;
    pause = false;
   
    
    counterTail = 1;
    allowMove = false;

    offset.x = screenWidth%SQUARE_SIZE;
    offset.y = screenHeight%SQUARE_SIZE;

    for (int i = 0; i < SNAKE_LENGTH; i++)
    {
        snake[i].position = (Vector2){ offset.x/2, offset.y/2 };
        snake[i].size = (Vector2){ SQUARE_SIZE, SQUARE_SIZE };
        snake[i].speed = (Vector2){ SQUARE_SIZE, 0 };

        if (i == 0) snake[i].color = DARKGREEN;
        else snake[i].color = BLUE;
    }

    for (int i = 0; i < SNAKE_LENGTH; i++)
    {
        snakePosition[i] = (Vector2){ 0.0f, 0.0f };
    }

    //fruits
    fruit.size = (Vector2){ SQUARE_SIZE, SQUARE_SIZE };
    fruit.color = GREEN;
    fruit.active = false;
    
    goldenFruit.size= (Vector2){ SQUARE_SIZE, SQUARE_SIZE };
    goldenFruit.color=YELLOW;
    goldenFruit.active=false;
    
    deadFruit.size= (Vector2){ SQUARE_SIZE, SQUARE_SIZE };
    deadFruit.color=BLACK;
    deadFruit.active=false;
    
    //murs
    recTab[0].x=2*SQUARE_SIZE + offset.x/2; //mur haut gauche
    recTab[0].y=2*SQUARE_SIZE + offset.y/2;
    recTab[0].width=(SQUARE_SIZE)-0.1;
    recTab[0].height=(SQUARE_SIZE*4)-0.1;
    
    recTab[1].x=3*SQUARE_SIZE + offset.x/2;
    recTab[1].y=2*SQUARE_SIZE + offset.y/2;
    recTab[1].width=(SQUARE_SIZE*3)-0.1;
    recTab[1].height=(SQUARE_SIZE)-0.1;
    
    recTab[2].x=screenWidth-(offset.x/2)-(3*SQUARE_SIZE); //mur haut droit
    recTab[2].y=2*SQUARE_SIZE + offset.y/2;
    recTab[2].width=(SQUARE_SIZE)-0.1;
    recTab[2].height=(SQUARE_SIZE*4)-0.1;
    
    recTab[3].x=screenWidth-(offset.x/2)-(6*SQUARE_SIZE); 
    recTab[3].y=2*SQUARE_SIZE + offset.y/2;
    recTab[3].width=(SQUARE_SIZE*3)-0.1;
    recTab[3].height=(SQUARE_SIZE)-0.1;
    
    recTab[4].x=2*SQUARE_SIZE + offset.x/2; //mur bas gauche
    recTab[4].y=screenHeight-(offset.y/2)-(6*SQUARE_SIZE);
    recTab[4].width=(SQUARE_SIZE)-0.1;
    recTab[4].height=(SQUARE_SIZE*3)-0.1;
    
    recTab[5].x=2*SQUARE_SIZE + offset.x/2; 
    recTab[5].y=screenHeight-(offset.y/2)-(3*SQUARE_SIZE);
    recTab[5].width=(SQUARE_SIZE*4)-0.1;
    recTab[5].height=(SQUARE_SIZE)-0.1;
    
    recTab[6].x=screenWidth-(offset.x/2)-(3*SQUARE_SIZE); //mur bas droit
    recTab[6].y=screenHeight-(offset.y/2)-(6*SQUARE_SIZE);
    recTab[6].width=(SQUARE_SIZE)-0.1;
    recTab[6].height=(SQUARE_SIZE*4)-0.1;
    
    recTab[7].x=screenWidth-(offset.x/2)-(6*SQUARE_SIZE);
    recTab[7].y=screenHeight-(offset.y/2)-(3*SQUARE_SIZE);
    recTab[7].width=(SQUARE_SIZE*3)-0.1;
    recTab[7].height=(SQUARE_SIZE)-0.1;
    
    highscore=LoadStorageValue(0);
    
    //sons
    fruitSound = LoadSound("SnakeSounds\\coin.wav");         // Load audio files
    goldenFruitSound = LoadSound("SnakeSounds\\sound.wav");  
    deathSound = LoadSound("SnakeSounds\\tanatana.ogg"); 
    music = LoadMusicStream("SnakeSounds\\mini1111.xm");
}

void drawWalls(){
    //mur haut gauche
    DrawRectangleRec(recTab[0],DARKBROWN);
    DrawRectangleRec(recTab[1],DARKBROWN);
    
    //mur haut droit
    DrawRectangleRec(recTab[2],DARKBROWN);
    DrawRectangleRec(recTab[3],DARKBROWN);
 
    //mur bas gauche
    DrawRectangleRec(recTab[4],DARKBROWN);
    DrawRectangleRec(recTab[5],DARKBROWN);
    
    //mur bas droit
    DrawRectangleRec(recTab[6],DARKBROWN);
    DrawRectangleRec(recTab[7],DARKBROWN);
        
}

bool checkCollisionMur(Vector2 position){
    bool res = false;
    for(int i=0; i<NUMBEROFWALL;i++){
        if(CheckCollisionPointRec(position, recTab[i])) res=true;
    }
    return res;
}

// Update game (one frame)
void UpdateGame(void)
{
    if (!gameOver)
    {
        if (IsKeyPressed('P')) pause = !pause;
        
        if (!pause)
        {
            UpdateMusicStream(music);
            
            // Player control
            if (IsKeyPressed(KEY_RIGHT) && (snake[0].speed.x == 0) && allowMove)
            {
                snake[0].speed = (Vector2){ SQUARE_SIZE, 0 };
                allowMove = false;
            }
            if (IsKeyPressed(KEY_LEFT) && (snake[0].speed.x == 0) && allowMove)
            {
                snake[0].speed = (Vector2){ -SQUARE_SIZE, 0 };
                allowMove = false;
            }
            if (IsKeyPressed(KEY_UP) && (snake[0].speed.y == 0) && allowMove)
            {
                snake[0].speed = (Vector2){ 0, -SQUARE_SIZE };
                allowMove = false;
            }
            if (IsKeyPressed(KEY_DOWN) && (snake[0].speed.y == 0) && allowMove)
            {
                snake[0].speed = (Vector2){ 0, SQUARE_SIZE };
                allowMove = false;
            }

            // Snake movement
            for (int i = 0; i < counterTail; i++) snakePosition[i] = snake[i].position;

            if ((framesCounter%5) == 0)
            {
                for (int i = 0; i < counterTail; i++)
                {
                    if (i == 0)
                    {
                        snake[0].position.x += snake[0].speed.x;
                        snake[0].position.y += snake[0].speed.y;
                        allowMove = true;
                    }
                    else snake[i].position = snakePosition[i-1];
                }
            }
            
            //collison mur
            if(checkCollisionMur(snake[0].position)){
                    gameOver = true;
                    PlaySound(deathSound);
            } 
            
            //teleport wall
            if((snake[0].position.x)>(screenWidth-offset.x)){
                snake[0].position = (Vector2){ offset.x/2, snake[0].position.y };
            }else if((snake[0].position.x)<0){
                snake[0].position = (Vector2){ screenWidth-(offset.x/2)-SQUARE_SIZE, snake[0].position.y };
            }
            else if((snake[0].position.y)>(screenHeight-offset.y)){
                snake[0].position = (Vector2){ snake[0].position.x, offset.y/2 };
            }else if((snake[0].position.y)<0){
                snake[0].position = (Vector2){ snake[0].position.x, screenHeight-(offset.y/2)-SQUARE_SIZE };
            }
            

            // Collision with yourself
            for (int i = 1; i < counterTail; i++)
            {
                if ((snake[0].position.x == snake[i].position.x) && (snake[0].position.y == snake[i].position.y)){
                    gameOver = true;
                    PlaySound(deathSound);
                }
            }


            // Fruit position calculation
            if (!fruit.active)
            {
                goldenFruit.position=(Vector2) {-100,-100}; //disparition du goldenFruit
                deadFruit.position=(Vector2) {-100,-100}; //disparition du deadFruit
                fruit.active = true;                
                fruit.position = (Vector2){ GetRandomValue(0, (screenWidth/SQUARE_SIZE) - 1)*SQUARE_SIZE + offset.x/2, GetRandomValue(0, (screenHeight/SQUARE_SIZE) - 1)*SQUARE_SIZE + offset.y/2 };

                for (int i = 0; i < counterTail; i++)
                {
                    while (((fruit.position.x == snake[i].position.x) && (fruit.position.y == snake[i].position.y))||checkCollisionMur(fruit.position))
                    {
                        fruit.position = (Vector2){ GetRandomValue(0, (screenWidth/SQUARE_SIZE) - 1)*SQUARE_SIZE + offset.x/2, GetRandomValue(0, (screenHeight/SQUARE_SIZE) - 1)*SQUARE_SIZE + offset.y/2 };
                        i = 0;
                    }
                } 
                
                //autres fruits
                if(counterTail%4==0){            
                    goldenFruit.position = (Vector2){ GetRandomValue(0, (screenWidth/SQUARE_SIZE) - 1)*SQUARE_SIZE + offset.x/2, GetRandomValue(0, (screenHeight/SQUARE_SIZE) - 1)*SQUARE_SIZE + offset.y/2 };
                    for (int i = 0; i < counterTail; i++)
                    {
                        if (checkCollisionMur(goldenFruit.position)){ //permet de ne pas embourber les conditions du while
                            goldenFruit.position=fruit.position; //si le fruit est sur un mur alors on reste dans le while
                        }
                        while (((goldenFruit.position.x == snake[i].position.x) && (goldenFruit.position.y == snake[i].position.y))||((goldenFruit.position.x == fruit.position.x) && (goldenFruit.position.y == fruit.position.y)))
                        {
                            goldenFruit.position = (Vector2){ GetRandomValue(0, (screenWidth/SQUARE_SIZE) - 1)*SQUARE_SIZE + offset.x/2, GetRandomValue(0, (screenHeight/SQUARE_SIZE) - 1)*SQUARE_SIZE + offset.y/2 };
                            i = 0;
                        }
                    }
                }else if(counterTail%7==0){
                    deadFruit.position = (Vector2){ GetRandomValue(0, (screenWidth/SQUARE_SIZE) - 1)*SQUARE_SIZE + offset.x/2, GetRandomValue(0, (screenHeight/SQUARE_SIZE) - 1)*SQUARE_SIZE + offset.y/2 };
                    for (int i = 0; i < counterTail; i++)
                    {
                        if (checkCollisionMur(deadFruit.position)){ 
                            deadFruit.position=fruit.position; 
                        }
                        while (((deadFruit.position.x == snake[i].position.x) && (deadFruit.position.y == snake[i].position.y))||((deadFruit.position.x == fruit.position.x) && (goldenFruit.position.y == fruit.position.y)))
                        {
                            deadFruit.position = (Vector2){ GetRandomValue(0, (screenWidth/SQUARE_SIZE) - 1)*SQUARE_SIZE + offset.x/2, GetRandomValue(0, (screenHeight/SQUARE_SIZE) - 1)*SQUARE_SIZE + offset.y/2 };
                            i = 0;
                        }
                    }
                }    
            }

            // Collision fruit
            if ((snake[0].position.x < (fruit.position.x + fruit.size.x) && (snake[0].position.x + snake[0].size.x) > fruit.position.x) &&
                (snake[0].position.y < (fruit.position.y + fruit.size.y) && (snake[0].position.y + snake[0].size.y) > fruit.position.y))
            {
                snake[counterTail].position = snakePosition[counterTail - 1];
                snake[counterTail].color = fruit.color;
                counterTail += 1;
                fruit.active = false;
                snake[0].score+=100;
                PlaySound(fruitSound);
            }
            
            // Collision goldenFruit
            if ((snake[0].position.x < (goldenFruit.position.x + goldenFruit.size.x) && (snake[0].position.x + snake[0].size.x) > goldenFruit.position.x) &&
                (snake[0].position.y < (goldenFruit.position.y + goldenFruit.size.y) && (snake[0].position.y + snake[0].size.y) > goldenFruit.position.y))
            {
                snake[counterTail].position = snakePosition[counterTail - 1];
                snake[counterTail].color = goldenFruit.color;
                counterTail += 1;
                goldenFruit.position=(Vector2) {-100,-100}; //disparition du goldenFruit
                snake[0].score+=250;
                PlaySound(goldenFruitSound);
            }
            
            // Collision deadFruit
            if ((snake[0].position.x < (deadFruit.position.x + deadFruit.size.x) && (snake[0].position.x + snake[0].size.x) > deadFruit.position.x) &&
                (snake[0].position.y < (deadFruit.position.y + deadFruit.size.y) && (snake[0].position.y + snake[0].size.y) > deadFruit.position.y))
            {
                gameOver=true;
                PlaySound(deathSound);
            }

            framesCounter++;
        }
    }
    else
    {
        if (IsKeyPressed(KEY_ENTER))
        {
            InitGame();
            StopMusicStream(music);
            PlayMusicStream(music);
            gameOver = false;
        }
    }
}

// Draw game (one frame)
void DrawGame(void)
{
    BeginDrawing();

        ClearBackground(RAYWHITE);
        
        if (!gameOver)
        {
            // Draw grid lines
            for (int i = 0; i < screenWidth/SQUARE_SIZE + 1; i++)
            {
                DrawLineV((Vector2){SQUARE_SIZE*i + offset.x/2, offset.y/2}, (Vector2){SQUARE_SIZE*i + offset.x/2, screenHeight - offset.y/2}, LIGHTGRAY);
            }

            for (int i = 0; i < screenHeight/SQUARE_SIZE + 1; i++)
            {
                DrawLineV((Vector2){offset.x/2, SQUARE_SIZE*i + offset.y/2}, (Vector2){screenWidth - offset.x/2, SQUARE_SIZE*i + offset.y/2}, LIGHTGRAY);
            }

            // Draw snake
            for (int i = 0; i < counterTail; i++) DrawRectangleV(snake[i].position, snake[i].size, snake[i].color);

            //Draw walls
            drawWalls();

            // Draw fruit to pick
            DrawRectangleV(fruit.position, fruit.size, fruit.color);
            if(counterTail%4==0) DrawRectangleV(goldenFruit.position, goldenFruit.size, goldenFruit.color);
            if(counterTail%7==0) DrawRectangleV(deadFruit.position, deadFruit.size, deadFruit.color);
            
            //score et timer en temps reel
            DrawText(TextFormat("TIME: %.02f", (float)framesCounter/60), 10, 10, 20, BLACK);
            DrawText(TextFormat("SCORE: %04i",snake[0].score), 10, 30, 20, BLACK);

            if (pause) {
                DrawText("GAME PAUSED", screenWidth/2 - MeasureText("GAME PAUSED", 70)/2, screenHeight/2 - 120, 70, BLACK);
                DrawText(TextFormat("TIME: %.02f", (float)framesCounter/60), screenWidth/2 - MeasureText("TIME: 00.00", 60)/2, screenHeight - 140, 60, BLUE);
                DrawText(TextFormat("SCORE: %04i",snake[0].score ), screenWidth/2 - MeasureText("SCORE: 0000", 60)/2, screenHeight - 200, 60, BLUE);
            }
        }
        else {
            DrawText("PRESS [ENTER] TO PLAY AGAIN", screenWidth/2 - MeasureText("PRESS [ENTER] TO PLAY AGAIN", 40)/2, screenHeight - 200, 40, BLACK);
            DrawText(TextFormat("TIME: %.02f", (float)framesCounter/60), screenWidth/2 - MeasureText("TIME: 00.00", 60)/2, screenHeight/2 - 120, 60, BLUE);
            DrawText(TextFormat("SCORE: %04i",snake[0].score ), screenWidth/2 - MeasureText("SCORE: 0000", 60)/2, screenHeight/2 - 180, 60, BLUE);
            if(highscore<snake[0].score){
                SaveStorageValue(0,snake[0].score);
                highscore=LoadStorageValue(0);

            }
            DrawText(TextFormat("HIGHSCORE: %04i",highscore ), screenWidth/2 - MeasureText("HIGHSCORE: 0000", 60)/2, screenHeight - 100, 60, RED);
        }
    EndDrawing();
}

// Unload game variables
void UnloadGame(void)
{
    // TODO: Unload all dynamic loaded data (textures, sounds, models...)
    UnloadSound(fruitSound);
    UnloadSound(goldenFruitSound);
    UnloadSound(deathSound);
    CloseAudioDevice();
}

// Update and Draw (one frame)
void UpdateDrawFrame(void)
{
    UpdateGame();
    DrawGame();
}

